package Msm::XmlParse;

use Msm::XmlNode;
use strict;

sub new {
	my $self=shift;
	my $xml=shift;
	my $ms=shift;
	my $jcode=$ms->{JCode};
	my $hash={};

	bless $hash,$self;
	$hash->{'XMLENCODE'}='utf8';
	if($xml){
		$xml=$hash->deleteComment($xml);
		$hash->{XML}=$xml;
		$xml=$hash->parseHead($xml);
		$xml=$jcode->JCodeConvert($ms,$xml,'sjis',$hash->{'XMLENCODE'});
		$hash->{ROOT}=$self->parse($xml,undef);
	}

	return $hash;

}

sub deleteComment{
	my $self=shift;
	my $xml=shift;
	$xml =~s/<\!--.*?-->//gos;
	return $xml;
}

#
#	RootMsm::XmlNodeԂ
#
sub getRootNode(){
	my $self=shift;
	return $self->{ROOT};
}

sub setRootNode(){
	my $self=shift;
	my $rnode=shift;
	$self->{ROOT}=$rnode;;
}

sub parse{
	my $self=shift;
	my $xml=shift;
	my $upnode=shift;

	if($xml =~ /<(?:(?:(?!\!))(\/|)([^\s\/<>\!]+)([^>]*?)(\/|)|\!\[CDATA\[(.*?)\]\])>/os){
		if(defined($5) && $5 ne ''){
			$upnode->addValue('<![CDATA['.$5.']]>');
			$self->parse($',$upnode);
		}elsif($1 eq '' && $4 eq ''){
			if($` && ($upnode)){
				$upnode->addValue($`.$&);
			}
			#print "$2\n";
			my $newnode=new Msm::XmlNode($2,$upnode,$self->attribute_parse($3));
			$self->parse($',$newnode);
		}elsif($1 eq '/' && $4 eq ''){
			if($`){
				$upnode->addValue($`);
			}
			my $parentnode=$upnode->getParentNode();
			if($parentnode){
				if($2 ne $upnode->getNodeName()){
					die "parse error $&";
				}
				$parentnode->addValue((defined($upnode->getValue())?$upnode->getValue():'').(defined($&)?$&:''));
				$parentnode->pushNode($upnode);
				$self->parse($',$parentnode);
			}else{
				return $upnode;
			}
		}elsif($4 eq '/'){
			#print "$2/\n";
			if($`){
				$upnode->addValue($`.$&);
			}
			my $newnode=new Msm::XmlNode($2,$upnode,$self->attribute_parse($3));
			$upnode->pushNode($newnode);
			$self->parse($',$upnode);
		}
	}
}


sub attribute_parse{
	my $self=shift;
	my $text=shift;
	my $hash={};

	while($text=~/([^<>=\s]+)\s*?=\s*?("[^"]*"|'[^']*')/gs){
		my $name=$1;
		my $attr=$2;
		$attr=~s/^"(.*)"$/$1/;
		$attr=~s/^'(.*)'$/$1/;
		$hash->{$name}=$attr;
	}

	return $hash;
}

sub parseHead{
	my $self=shift;
	my $xml=shift;
	if($xml=~/^[^<]*<\?.*?\?>/){
		if($xml =~ /^[^<]*<\?.*?encoding=\"(.*?)\".*\?>/i){
			$self->{'XMLENCODE'}='utf8';
			if($1 =~ /EUC-JP/i){
				$self->{'XMLENCODE'}='euc';
			}
			elsif($1 =~ /UTF-8/i){
				$self->{'XMLENCODE'}='utf8';
			}
			elsif($1 =~ /SHIFT_JIS/i){
				$self->{'XMLENCODE'}='sjis';
			}
		}
		$xml=~s/^[^<]*<\?.*?\?>//;
		$xml=~s/<\!doctype.*?>//i;
	}else{
		die 'parse error nohead';
	}
	return $xml;
}

sub createXml{
	my $self=shift;
	my $ms=shift;
	my $jcode=$ms->{JCode};
	my $xml='<?xml version="1.0" encoding="utf-8"?>'."\n";
	$xml=$self->createNodeXml($xml,$self->getRootNode());

	#utf8ɃGR[h
	$xml=$jcode->JCodeConvert($ms,$xml,'utf8','sjis');

	return $xml;
}

sub createNodeXml{
	my $self=shift;
	my $xml=shift;
	my $node=shift;
	$xml.=$self->createtag($node);
	foreach my $n($node->getChildNodes()){
		$xml=$self->createNodeXml($xml,$n);
	}
	$xml.=$self->closetag($node);
	return $xml;
}

sub createtag{
	my $self=shift;
	my $node=shift;
	my $tag=undef;
	$tag='<'.$node->getNodeName().'';
	foreach($node->getAttrNames()){
		$tag.=' '.$_.'="'.$node->getAttr($_).'"';
	}
	unless($node->getChildNodes()){
		if($node->getValue() ne ''){
			$tag.='>'.Msm::XmlNode->escape($node->getValue()).'</'.$node->getNodeName().'>'."\n";
		}else{
			$tag.='/>'."\n";
		}
	}else{
		$tag.='>'."\n";
		$self->{closetag}.='</'.$node->getNodeName().'>'."\n";
	}

	return $tag;
}

sub closetag{
	my $self=shift;
	my $node=shift;
	if($node->getChildNodes()){
		return '</'.$node->getNodeName().'>'."\n";
	}
	return '';
}

1;
